// * This makes emacs happy -*-Mode: C++;-*-
/****************************************************************************
 * Copyright (c) 2002 Fabrica de Software XXI SRL - info@lafabrica.com.ar	*
 * Copyright (c) 2002 RaqLink SA - info@raqlink.com							*
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 *	Author Miguel Pinkas <miguel@lafabrica.com.ar> 							*
 ****************************************************************************/
#include "ndk++.h"

using namespace std;

namespace NDK_XX{

ofstream logfile("NDK_debug");
ofstream &get_logfile(){return logfile;}
//-----------------------------------------------------------------------
string NDKDialogBox::lblOk="  OK  ";
string NDKDialogBox::lblCancel="Cancel";
string NDKDialogBox::lblYes="  Yes ";	
string NDKDialogBox::lblNo="  No  ";
string NDKDialogBox::titDataChgd="Data changed!";
string NDKDialogBox::msgAskToSave="Save before we exit?";



//-----------------------------------------------------------------------
NDKMenu::NDKMenu (	ListOfItems &items, 
					int  lines, 	int  cols, 
					int  begin_y, int  begin_x,
					bool framed,        // Reserve space for a frame?
					bool own_items)  // Autocleanup of Items?
	: NCursesMenu(items.begin()
		,lines,cols
		,begin_y,begin_x
		,framed,own_items),have_frame(framed) 
{
	link_items(items);
}


//-----------------------------------------------------------------------
NDKMenu::NDKMenu( int  lines, int  cols, int  begin_y, int  begin_x
					,bool framed)
	: NCursesMenu(lines,cols,begin_y,begin_x),have_frame(framed) 
{
}


//-----------------------------------------------------------------------
void NDKMenu::link_items(ListOfItems &items)
{
	ListOfItems::const_iterator it;
	for(it = items.begin(); it !=items.end();it++){
		NDKMenuItem *pItem = dynamic_cast<NDKMenuItem *>(*it);
		if(!pItem)
			throw NDKDlgException<NDKMenu>(this,"No item or wrong item type"
									" in NDKMenu::link_items");
		pItem->pMenu = this;
	}
}


//-----------------------------------------------------------------------
void NDKMenu::lefttext(int row,const char *label) {
  if (label) {
    OnError(addstr(row, 0, label, width()));
  }
}


//-----------------------------------------------------------------------
void NDKMenu::righttext(int row,const char *label) {
  if (label) {
    int x = maxx() - strlen(label);
    if (x<0)
      x=0;
    OnError(addstr(row, x, label, width()));
  }
}


/************************************************************************
*  	class NDKMenuBar													*
************************************************************************/
//-----------------------------------------------------------------------
void NDKMenuBar::show_text(const string &t,char where){
	if (!t.empty()){
		switch(where){
			case 'l': lefttext(0,t.c_str()); break;
			case 'c': centertext(0,t.c_str()); break;
			case 'r': righttext(0,t.c_str()); break;
			default: centertext(0,t.c_str()); break;
		}
	}
}


//-----------------------------------------------------------------------
NDKMenuBar::NDKMenuBar (int rows, int cols,int y, int x
  					,ListOfItems &items
					,const char *ttitle, const char *btitle
					,const char *title_pos
					,char orient,bool framed)
    : 	NDKMenu(rows, cols, y, x,framed)
    	,orientation(orient)
		,title(ttitle),bTitle(btitle),titlePos(title_pos)
		,itemList(items)
{
	
	short count = items.size()-1;
	
	if(orientation=='v')  set_format(count,1);
	else set_format(1,count);

	InitMenu(items);
	
	link_items(items);
	
	if(framed)
		frame();

	//by default it's 2, but user may be clever enough to pass in "" ...
	while(titlePos.size()< 2)
		titlePos+='c';

	setTitle();
}


/************************************************************************
*  	class NDKSubMenu												*
************************************************************************/
//-----------------------------------------------------------------------
void NDKSubMenu::setItems(ListOfItems &items) 
{
	NDKMenuItem *pItem = dynamic_cast<NDKMenuItem *>(items[0]);
	ListOfItems::iterator it;
	if(pItem->get_menu())
		NDKMenu::setItems(items);
	else 
		InitMenu(items);
	refresh();
}
	

/************************************************************************
*  	class NDKPopUpMenu												*
************************************************************************/
//-----------------------------------------------------------------------
NDKPopUpMenu::~NDKPopUpMenu(){ 
	//logfile << "Pop up\t" << name() << endl;
	ListOfItems::iterator it;
	for(it=items.begin();it!=items.end();it++){
		delete *it;
	}
	
}  
	

//-----------------------------------------------------------------------
bool NDKPopUpMenu::action() {
	NDKSubMenu S(items.size()-1,largest_name_len()
					,submenuy(),submenux(),items,sub_menu_label.c_str());
	S();
	return FALSE; 
}
	

//-----------------------------------------------------------------------
short NDKPopUpMenu::submenux() const{
	// default x position: right where the items label is shown 
	// (this works wether we have a normal curses menu 
	// or a menu bar with h layout
	short x = pMenu->begx()
			+ pMenu->get_itemmarklen() 
			+ pMenu->is_framed();
	
	NDKMenuBar *pParent = dynamic_cast<NDKMenuBar *>(pMenu);
	if(pParent && pParent->get_orientation() == 'h')
		x += (index() * pMenu->get_itemwidth());

	return x;
}


//-----------------------------------------------------------------------
short NDKPopUpMenu::submenuy()const{
	short y = pMenu->begy() + pMenu->is_framed();

	NDKMenuBar *pParent = dynamic_cast<NDKMenuBar *>(pMenu);
	if(pParent && pParent->get_orientation() == 'v'){
		y += index();
	}
	return y;
}

		
//-----------------------------------------------------------------------
int NDKPopUpMenu::largest_name_len()const{
	unsigned MaximumNameLength = 0;
	ListOfItems::const_iterator it;
	for(it = items.begin(); it !=items.end();it++){
		NDKMenuItem *pItem = dynamic_cast<NDKMenuItem *>(*it);
		ITEM *item = pItem->get_curses_item();
		if (item && item->name.length > MaximumNameLength ){
			MaximumNameLength  = item->name.length;
		}
	}
	if(sub_menu_label.size() > MaximumNameLength)
		MaximumNameLength = sub_menu_label.size();
	return MaximumNameLength;
}


/************************************************************************
*  	class NDKDlgControl												*
************************************************************************/
//-----------------------------------------------------------------------
BEGIN_FIRST_BINDINGS(NDKDlgControl)
//	ON_NOTIFY_RANGE(LI_SEL_CHANGE,0,0xffff,NDKDlgControl::on_sel_change)
END_BINDINGS()


ofstream &NDKDlgControl::send_to_log(const string &what)
{
	logfile << what;
	return logfile;
}


//-----------------------------------------------------------------------
bool NDKDlgControl::process_message(NDKDlgControl *pTarget,int what
											, int ctrl_id, void* pExtra)
{
	const BINDING_MAP* binding_map;
	const BINDING* pEntry;

	for (binding_map = pTarget->get_binding_map(); binding_map != 0;
	  binding_map = binding_map->base_map)
	{
		assert(bindings_map != binding_map->base_map);

		pEntry = get_binding(binding_map->pEntries, what, ctrl_id);
		if (pEntry)
		{	
			if(what==0)
			(pTarget->*pEntry->p_cmd_fn)(); 
			else// found it
			(pTarget->*pEntry->p_notif_fn)(ctrl_id); 
			return true;
		}
	}
	return false;   // not handled
}


//-----------------------------------------------------------------------
const BINDING* NDKDlgControl::get_binding(const BINDING* pEntry,int what, int ctrl_id)
{
	// C version of search routine
	while (pEntry->pfn())
	{
		if (pEntry->what == what &&
			ctrl_id >= pEntry->ctrl_id && ctrl_id <= pEntry->last_ctrl_id){
			return pEntry;
		}
		pEntry++;
	}
	return 0;    // not found
}


//-----------------------------------------------------------------------
NDKDlgControl::styles 
NDKDlgControl::set_styles(NDKDlgControl::styles s)
{ 
	int old_style=field_style;
	field_style=s;
	return styles(old_style);
}
	

//-----------------------------------------------------------------------
NDKDlgControl::styles 
NDKDlgControl::set_style(NDKDlgControl::styles s,bool on)
{ 
	int old_style=field_style;
	if (on)	field_style|=s;
	else 	field_style&=~s;
	return styles(old_style);
}  


//-----------------------------------------------------------------------
NDKDlgControl::states 
NDKDlgControl::set_states(NDKDlgControl::states s)
{ 
	int old_state=field_state;
	field_state=s;
	return states(old_state);
}
	

//-----------------------------------------------------------------------
NDKDlgControl::states 
NDKDlgControl::set_state(NDKDlgControl::states s,bool on)
{ 
	int old_state=field_state;
	if (on)	field_state|=s;
	else 	field_state&=~s;
	return states(old_state);
}  


//-----------------------------------------------------------------------
int NDKDlgControl::mvwin(int y, int x)
{
	if(!is_moveable())
		return E_REQUEST_DENIED;
		
	int result=0;
	int new_x=begx()+x;
	int new_y=begy()+y;
	
	if(	new_x > pParent->begx() 
		&& new_x+ width() <= pParent->begx()+pParent->maxx() 
		&& new_y > pParent->begy() 
		&& new_y+ height() <= pParent->begy()+pParent->maxy()
	){
		result = NCursesPanel::mvwin(new_y,new_x);
		if(result==OK){
			w->_begx = new_x;
			w->_begy = new_y;
			refresh();
		}
	}
	return result;
}

		
//-----------------------------------------------------------------------
void NDKDlgControl::activate()
{
	top();
	set_state(active);
	if(has_frame()){
		attron(A_BOLD);
		box();
		attroff(A_BOLD);
		refresh();
	}
}


//-----------------------------------------------------------------------
void NDKDlgControl::deactivate()
{
	set_state(active,false);
	if(has_frame()){
		attroff(A_BOLD);
		box();
		refresh();
	}
}


/************************************************************************
*  	class NDKDlgLabel												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgLabel::NDKDlgLabel( NDKDlgControl *parent
								,int id, const string &title
								,int lines,int cols,int y, int x
								,int style)
	: NDKDlgControl(parent,id,lines,cols,y,x,style)
		,text(title)
		,pos_x(0),pos_y(0)
		,viewport(this)
		,cur_x(0),cur_y(0),x_start(0)
{
	on_initial_update=true;
	if(has_frame()){
		viewport = new NCursesWindow(*this);
		pos_x++; 
		pos_y++; 
	}
	set_text(title);
	on_initial_update=false;
}


//-----------------------------------------------------------------------
NDKDlgLabel::NDKDlgLabel( int id, const string &title
								,int lines,int cols,int y, int x
								,int style)
	: NDKDlgControl(id,lines,cols,y,x,style)
		,text(title)
		,pos_x(0),pos_y(0)
		,viewport(this)
		,cur_x(0),cur_y(0)
{
	on_initial_update=true;
	if(has_frame()){
		viewport = new NCursesWindow(*this);
		pos_x++; 
		pos_y++; 
	}
	set_text(title);
	on_initial_update=false;
}


//-----------------------------------------------------------------------
void NDKDlgLabel::set_text(const string &v)
{
	string old_text= text;
	text = v;

	clear_text();
	viewport->addstr(cur_y,cur_x, text.c_str(), max_len());
	viewport->move(cur_y,cur_x);
	viewport->cursyncup();
	viewport->refresh();
	
	if(old_text!=text && notify_enabled()){
		process_message(pParent,TXT_CHANGED,get_ctrl_id(),0);
	}
}
	
	
//-----------------------------------------------------------------------
int NDKDlgLabel::driver(int key)
{
	switch(key) {
		case KEY_UP:	mvwin(-1,0);	return TRUE;
		case KEY_DOWN:	mvwin(1,0);		return TRUE;
		case KEY_LEFT:	mvwin(0,-1);	return TRUE;
		case KEY_RIGHT:	mvwin(0,1);		return TRUE;
		default:					break;
	}
	return FALSE;
}


//-----------------------------------------------------------------------
void NDKDlgLabel::activate()
{
	NDKDlgControl::activate();
	if(!has_frame()){
		viewport->addch(cur_y,cur_x,text[0]|A_BOLD|A_UNDERLINE);
		viewport->move(cur_y,cur_x);
		viewport->cursyncup();
		viewport->refresh();
	}
}


//-----------------------------------------------------------------------
void NDKDlgLabel::deactivate()
{
	NDKDlgControl::deactivate();
	if(!has_frame()){
		viewport->addch(cur_y,cur_x,text[0]);
		viewport->move(cur_y,cur_x);
		viewport->cursyncup();
		viewport->refresh();
	}
}


//-----------------------------------------------------------------------
void NDKDlgLabel::clear_text(bool ref=false)
{
	for(cur_x=viewport->maxx();cur_x; cur_x--){
		viewport->addch(cur_y,cur_x,getbkgd());
	}
	viewport->addch(cur_y,cur_x,getbkgd());
}


/************************************************************************
*  	class NDKDlgButton												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgButton::NDKDlgButton( NDKDlgControl *parent
								,int id,const string &title
								,int lines,int cols,int y, int x
								,int style)
		:NDKDlgLabel(parent,id,title,lines,cols,y,x,style)
{
}


//-----------------------------------------------------------------------
int NDKDlgButton::driver(int key)
{
	int result= FALSE;
	switch(key) {
		case 10: 	result = action();  break;
 		default:	result = NDKDlgLabel::driver(key); break; 
	}
	return result;
}


//-----------------------------------------------------------------------
void NDKDlgButton::activate()
{
	NDKDlgControl::activate();
	attron(A_BOLD);
	set_text(text);
	
}


//-----------------------------------------------------------------------
void NDKDlgButton::deactivate()
{
	NDKDlgControl::activate();
	attroff(A_BOLD);
	set_text(text);
}


/************************************************************************
*  	class NDKDlgPushButton												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgPushButton::NDKDlgPushButton( NDKDlgControl *parent
								,int id, const string &title
								,int lines,int cols,int y, int x
								,int style)
		:NDKDlgButton(parent,id, title,lines,cols,y,x,(style&=~framed))
{
	on_initial_update=true;
	set_style(framed,true);
	set_text(text);
	on_initial_update=false;
}


//-----------------------------------------------------------------------
NDKDlgPushButton::NDKDlgPushButton( NDKDlgControl *parent
								,int id, const string &title
								,int y, int x)
		:NDKDlgButton(parent,id,title,1,title.size()+3,y,x,tabstop|notify)
{
	on_initial_update=true;
	set_style(framed,true);
	set_text(text);
	on_initial_update=false; 
}


//-----------------------------------------------------------------------
void NDKDlgPushButton::set_text(const string &v)
{
	int max_len = width()-pos_x-has_frame();
	string new_text= v.substr(0, max_len); 
	text = new_text;
	clrtoeol();	
	centertext(0,text.c_str());
	if(has_frame()){
		addch(0,0, '[');
		addch(0,maxx(), ']');
	}
	move(0,0);
	refresh();
}
	
	

/************************************************************************
*  	class NDKDlgListItemData											*
************************************************************************/
//----------------------------------------------------------------------//
//		NDKDlgListItemData::operator=									//
//																		//
//		copy from the visible item to the data buffer only gets			//
//		the style and the state of the visible item, since we don't		//
//		allow text or framing changes.									//
//																		//
//----------------------------------------------------------------------//
NDKDlgListItemData &
NDKDlgListItemData::operator=(const NDKDlgListItem &rha)
{
	set_styles(rha.get_style());
	set_states(rha.get_state());
	set_default(rha.is_default());
	return *this;
}


//-----------------------------------------------------------------------
NDKDlgListItemData &
NDKDlgListItemData::operator=(const NDKDlgListItemData &rha)
{
	set_text(rha.get_text());
	set_ctrl_id(rha.get_ctrl_id());
	set_styles(rha.get_style());
	set_states(rha.get_state());
	set_default(rha.is_default());
	set_user_data(rha.get_user_data());
	return *this;
}


//-----------------------------------------------------------------------
int
NDKDlgListItemData::set_state(int s,bool on)
{ 
	int old_state=field_state;
	if (on)	field_state|=s;
	else 	field_state&=~s;
	return old_state;
}  


/************************************************************************
*  	class NDKDlgListItem												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgListItem::NDKDlgListItem( NDKDlgControl *parent
								,int id, const string &title
								,int lines,int cols,int y, int x
								,int style)
		:NDKDlgButton(parent,id, title,lines,cols,y,x,style),user_data(0)
{
}


//-----------------------------------------------------------------------
int NDKDlgListItem::driver(int key)
{
	int result= FALSE;
	switch(key) {
		case KEY_ENTER: 
		case ' ': 
			set_check(!is_checked());  
			result=TRUE;
			break;
		default:
			break;
	}
	return result;
}


//-----------------------------------------------------------------------
void NDKDlgListItem::set_text(const string &v)
{
	string new_text= v; //.substr(0,width()); 
	text = new_text;
	clear_text();
	show_state();
	addstr(pos_y,state_size(),text.c_str(),width()-state_size());
	move(0,0);
	refresh();
}


//-----------------------------------------------------------------------
NDKDlgListItem &NDKDlgListItem::operator=(const NDKDlgListItemData &rha)
{
	set_text(rha.get_text());
	set_ctrl_id(rha.get_ctrl_id());
	set_styles(styles(rha.get_style()));
	set_states(states(rha.get_state()));
	set_default(rha.is_default());
	set_user_data(rha.get_user_data());
	return *this;
}


/************************************************************************
*  	class NDKDlgRadioButton												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgRadioButton::NDKDlgRadioButton( NDKDlgControl *parent
								,int id,const string &title
								,int lines,int cols
								,int y, int x,int style)
	:NDKDlgListItem(parent,id,title,lines,cols,y,x,style)
{	
	on_initial_update=true;
	set_text(text);
	on_initial_update=false;
}


//-----------------------------------------------------------------------
NDKDlgRadioButton::NDKDlgRadioButton( NDKDlgControl *parent
								,int id, const string &title
								,int y, int x)
	:NDKDlgListItem(parent,id,title,1,title.size()+4,y,x,tabstop|notify)
{
	on_initial_update=true;
	set_text(text);
	on_initial_update=false;
}


/************************************************************************
*  	class NDKDlgCheckButton											*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgCheckButton::NDKDlgCheckButton( NDKDlgControl *parent
								,int id,const string &title
								,int lines,int cols
								,int y, int x,int style)
		:NDKDlgListItem(parent,id,title,lines,cols,y,x,style)
{
	on_initial_update=true;
	set_text(text);
	on_initial_update=false;
}


//-----------------------------------------------------------------------
NDKDlgCheckButton::NDKDlgCheckButton( NDKDlgControl *parent
								,int id,const string &title
								,int y, int x)
	:NDKDlgListItem(parent,id,title,1,title.size()+4,y,x,tabstop|notify)
{
	on_initial_update=true;
	set_text(text);
	on_initial_update=false;
}


/************************************************************************
*  	class NDKDlgEdit												*
************************************************************************/
//-----------------------------------------------------------------------
NDKDlgEdit::NDKDlgEdit( NDKDlgControl *parent
								,int id,const string &title
								,int lines,int cols,int y, int x
								,int style)
		:NDKDlgLabel(parent,id,title,lines,cols,y,x,style)
{
}


NDKDlgEdit::~NDKDlgEdit()
{
	if(has_frame())
		delete viewport;
}


//-----------------------------------------------------------------------
int NDKDlgEdit::driver(int key)
{
	if(edit(key))
		return TRUE;
	return NDKDlgLabel::driver(key);
}


//-----------------------------------------------------------------------
void NDKDlgEdit::add_char(int key)
{
	text+=key;
	viewport->addch(cur_y,cur_x,key);
}


//-----------------------------------------------------------------------
void NDKDlgEdit::insert_char(int key)
{
	if(text.size()+1 < max_len()){
		text.insert(cur_x,1,(char)key);
		viewport->insch(cur_y,cur_x,key);
	}
}


//-----------------------------------------------------------------------
int NDKDlgEdit::add_or_insert_char(int key)
{
	if(!isprint(key))
		return FALSE;
		
	if(cur_x > -1 && cur_x < viewport->width()){
		if(overwrite())add_char(key);
		else insert_char(key);
		viewport->move(cur_y,++cur_x);
		viewport->cursyncup();
		viewport->refresh();
		if(notify_enabled())
			process_message(pParent,TXT_CHANGED,get_ctrl_id(),0);
	}
	return TRUE;
}


//-----------------------------------------------------------------------
int NDKDlgEdit::edit(int key)
{
//	getyx(cur_y,cur_x);

	switch(key) {
		case KEY_BACKSPACE:
		case KEY_DC:
			if(cur_x){
				text.erase(--cur_x,1);
				viewport->delch(cur_y,cur_x);
				viewport->move(cur_y,cur_x);
				viewport->cursyncup();
				viewport->refresh();
			}
			return TRUE;
		case KEY_LEFT:
			if(cur_x){
				viewport->move(cur_y,--cur_x);
				viewport->cursyncup();
				viewport->refresh();
			}
			return TRUE;
		case KEY_RIGHT:
			if(cur_x < visible_size()){
				viewport->move(cur_y,++cur_x);
				viewport->cursyncup();
				viewport->refresh();
			}
			return TRUE;
		default:
			if(add_or_insert_char(key))	
				return TRUE;
	}
	return FALSE;
}


/************************************************************************
*  	class NDKDlgListBox													*
************************************************************************/
//-----------------------------------------------------------------------

NDKDlgListBox::NDKDlgListBox( NDKDlgControl *parent
					,int id,const string &title
					,int lines,int cols,int y, int x
					,int style)
		:NDKDlgControl( parent,id,lines,cols,y,x,style)
			,pos_x(0), pos_y(0)
			,curr_item(-1)
			,top_item(-1)
			,bottom_item(-1)
			,cur_sel(-1)
{
	on_initial_update=true;
	if(has_frame()){
		pos_x++; 
		pos_y++; 
	}
	set_text(title);
	draw_scrollbar();
	on_initial_update=false;
}


//-----------------------------------------------------------------------
BEGIN_BINDINGS(NDKDlgListBox,NDKDlgControl)
	ON_NOTIFY_RANGE(LI_SEL_CHANGE,0,0xffff,NDKDlgListBox::on_sel_change)
END_BINDINGS()


//-----------------------------------------------------------------------

NDKDlgListBox::~NDKDlgListBox()
{ 
	hide();
	VisibleItems::iterator it;
	for(it=items.begin();it!=items.end();it++){
//		logfile << "Deleted Visible item " << (*(it))->get_text() << endl;
		delete *it;
	}
	
	delete_data();
}


//-----------------------------------------------------------------------
void NDKDlgListBox::delete_data()
{
	ListItems::iterator d;
	for(d=data.begin();d!=data.end();d++){
//		logfile << "Deleted data item " << (*(d))->get_text() << endl;
		delete *d;
	}
}


//-----------------------------------------------------------------------
void NDKDlgListBox::clear()
{
	delete_data();
	data.clear();
	curr_item=top_item=bottom_item=0;
	refresh_items();
	show_empty();
	draw_scrollbar();
	refresh();
}


//-----------------------------------------------------------------------
void NDKDlgListBox::show_empty()
{
	if(data.empty()){
		cur_sel=-1;
		items[0]->set_text("Empty");
		items[0]->enable(false);
	}

}


//-----------------------------------------------------------------------
void NDKDlgListBox::draw_scrollbar()
{
	if(!has_vscrollbar())
		return;
    int i, row, col, h_len, v_len;
    int winheight=items.size();
    int dataheight = data.size();
    int top = ((top_item==-1)? 0:top_item);

	if(dataheight==0)
		dataheight = 1;
		
    v_len = (winheight*winheight + dataheight - 1)/dataheight;
    if (v_len==0)
      v_len = 1;
    if (v_len > winheight)
      v_len = winheight;


    row  = (top * winheight + dataheight - 1) / dataheight;
    if (row + v_len > winheight)
      row = winheight - v_len;

    vline(1,maxx(),winheight);
    attron(A_REVERSE);
    if (v_len>=2) {
      addch(row+1,maxx(),ACS_UARROW);
      for(i=2;i<v_len;i++)
		addch(row+i,maxx(),' ');
      addch(row+v_len,maxx(),ACS_DARROW);
    }
    else {
      for(i=1;i<=v_len;i++)
		addch(row+i,maxx(),' ');
    }
    attroff(A_REVERSE);
}
//-----------------------------------------------------------------------

int NDKDlgListBox::add_item(const string &new_item,int pos
								,bool is_default= false,bool make_visible)
{
	NDKDlgListItemData *item=new NDKDlgListItemData(new_item,data.size());
	item->set_default(is_default);
	add_item(item,pos,make_visible);
}


//-----------------------------------------------------------------------

int NDKDlgListBox::add_item(NDKDlgListItemData *new_item,int pos
													,bool make_visible)
{
	int old_current = curr_item;
	///by default, we insert at the end
	ListItems::iterator where=data.end();
	
	if(pos!=-1 ){
		if(pos >-1 && pos < data.size()){
			where=data.begin()+pos;
			if(make_visible){
				curr_item=pos;
			}
		}
		else 
			throw 
				NDKDlgException<NDKDlgListBox>(this
									,"Position out of range in add_item");
	}
	else{
		if(data.empty()==false){/// if no data, curr_item already is 0
			if(make_visible){
				curr_item++;
			}
		}
	}
	data.insert(where,new_item);
	adjust_top_and_bottom();
	draw_scrollbar();
	refresh();
}


//-----------------------------------------------------------------------

int NDKDlgListBox::remove_item(int pos)
{
	if(pos>=0 && pos < data.size()){
		data.erase(data.begin()+pos);
		if(pos<= curr_item && curr_item)
			curr_item--;
		if(cur_sel == pos)
			cur_sel = -1;
			
			
		logfile << "remove_item: cursel=" << cur_sel << endl;
		adjust_top_and_bottom();
		draw_scrollbar();
		show_empty();
		refresh();
	}
};


//-----------------------------------------------------------------------

int NDKDlgListBox::refresh()
{
	for( int idx=top_item;idx <= bottom_item; idx++){
		if(idx!=curr_item)
			deactivate_item(idx);
	}
	activate_item(curr_item);
}


//-----------------------------------------------------------------------

void NDKDlgListBox::refresh_items()
{
	for( int idx=top_item;idx <= bottom_item; idx++){
		update_item(idx);
	}
	clear_items();

}


//-----------------------------------------------------------------------
void NDKDlgListBox::clear_items()
{
	for( int idx=data.size()-top_item;idx <items.size(); idx++){
		items[idx]->set_text("");
	}
}


//-----------------------------------------------------------------------

int NDKDlgListBox::driver (int key) 
{
	if(	data.size() 
		&& curr_item!=-1 
		&& visible_item(curr_item)->driver(key)==TRUE)
		return TRUE;
	switch(key) {
		case KEY_UP:	move_up();	return TRUE;
		case KEY_DOWN:	move_down();return TRUE;
		case KEY_LEFT:	move_left();return TRUE;
		case KEY_RIGHT:	move_right();return TRUE;
		default: break;
	}
	return FALSE;
}


//-----------------------------------------------------------------------

void NDKDlgListBox::adjust_top_and_bottom()
{
	
	if(curr_item < top_item){
		top_item = curr_item;
		bottom_item = top_item + (items.size()-1);
	}
	if(curr_item > bottom_item){
		bottom_item = curr_item;
		top_item = bottom_item - (items.size()-1);
		if(top_item < 0) 
			top_item =0;
	}
	
	if(top_item > data.size()) top_item = (data.size()-1);
	if(bottom_item > data.size()) bottom_item = (data.size()-1);
		
	if(top_item < 0) top_item =0;
	if(bottom_item < 0) bottom_item =0;
	
	refresh_items();
}


//-----------------------------------------------------------------------

void NDKDlgListBox::activate()
{
	NDKDlgControl::activate();
	attron(A_BOLD);
	set_text(_title);
	attroff(A_BOLD);
	draw_scrollbar();
	refresh();
}


//-----------------------------------------------------------------------

void NDKDlgListBox::deactivate()
{
	NDKDlgControl::deactivate();
	attron(A_BOLD);
	set_text(_title);
	attroff(A_BOLD);
	deactivate_item(curr_item);
	draw_scrollbar();
 	refresh();
}


//-----------------------------------------------------------------------

void NDKDlgListBox::show_text(const string &t,char where)
{
	if (!t.empty()){
		switch(where){
			case 'l': lefttext(0,t.c_str()); break;
			case 'c': centertext(0,t.c_str()); break;
			case 'r': righttext(0,t.c_str()); break;
			default: centertext(0,t.c_str()); break;
		}
	}
}


//-----------------------------------------------------------------------

void NDKDlgListBox::lefttext(int row,const char *label) {
  if (label) {
    OnError(addstr(row, 1, label, width()));
  }
}


//-----------------------------------------------------------------------

void NDKDlgListBox::righttext(int row,const char *label) {
  if (label) {
    int x = maxx() - strlen(label)-1;
    if (x<0)
      x=0;
    OnError(addstr(row, x, label, width()));
  }
}


//-----------------------------------------------------------------------

void NDKDlgListBox::activate_item(int item_idx)
{
	update_item(item_idx);
	visible_item(item_idx)->activate();
}


//-----------------------------------------------------------------------

void NDKDlgListBox::deactivate_item(int item_idx)
{
	visible_item(item_idx)->deactivate();
	update_data(item_idx);
}


//-----------------------------------------------------------------------

void NDKDlgListBox::update_data(int item_idx)
{
	if(data.empty()||item_idx<0||item_idx >= data.size() )
		return;
	NDKDlgListItem &itm= *(visible_item(item_idx)); 
	NDKDlgListItemData &dat = *(data[item_idx]);
	dat = itm;
	
}


//-----------------------------------------------------------------------

void NDKDlgListBox::update_item(int item_idx)
{
	if(data.empty()||item_idx<0||item_idx >= data.size() )
		return;
	NDKDlgListItem &itm= *(visible_item(item_idx));
	NDKDlgListItemData &dat = *(data[item_idx]);
	itm = dat;
}


//-----------------------------------------------------------------------

void NDKDlgListBox::move_up()
{
	if(curr_item){
		curr_item--;
		adjust_top_and_bottom();
		draw_scrollbar();
		refresh();
		if(notify_enabled()){
			process_message(pParent,LI_POS_CHANGE,get_ctrl_id(),0);
		}
	}
}


//-----------------------------------------------------------------------

void NDKDlgListBox::move_down()
{
	if(data.size() && curr_item < data.size()-1){
		curr_item++;
		adjust_top_and_bottom();
		draw_scrollbar();
		refresh();
		if(notify_enabled()){
			process_message(pParent,LI_POS_CHANGE,get_ctrl_id(),0);
		}
	}
}


//-----------------------------------------------------------------------

void NDKDlgListBox::move_left()
{
}


//-----------------------------------------------------------------------

void NDKDlgListBox::move_right()
{
}


//-----------------------------------------------------------------------
string NDKDlgListBox::get_item_text(int pos)
{
	string result;
	if(pos>-1 && pos < data.size())
		result = data[pos]->get_text();

	return result;
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_item_text(int pos,const string &txt)
{
	if(pos>-1 && pos < data.size())
		data[pos]->set_text(txt);
}


//-----------------------------------------------------------------------
void *NDKDlgListBox::get_item_data(int pos)
{
	void *result=0;
	if(pos>-1 && pos < data.size())
		result = data[pos]->get_user_data();

	return result;
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_item_data(int pos,void *usr_data)
{
	if(pos>-1 && pos < data.size())
		data[pos]->set_user_data(usr_data);
}


//-----------------------------------------------------------------------
NDKDlgListItemData *NDKDlgListBox::get_item(int pos)
{
	NDKDlgListItemData *result=0;
	if(pos>-1 && pos < data.size())
		result = data[pos];

	return result;
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_item(int pos,NDKDlgListItemData *item)
{
	if(pos>-1 && pos < data.size()){
		delete data[pos];
		data[pos]=item;
		refresh_items();
		refresh();
	} 
}


//-----------------------------------------------------------------------
int NDKDlgListBox::get_cur_pos()
{
	return curr_item;	
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_cur_pos(int pos)
{
	if(pos>-1 && pos < data.size()){
		curr_item = pos;
		adjust_top_and_bottom();
		draw_scrollbar();
		refresh();
	}
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_cur_sel(int pos)
{
	if(pos>-1 && pos < data.size()){
		curr_item = pos;
		adjust_top_and_bottom();
		draw_scrollbar();
		visible_item(pos)->set_check(true);
		check_multisel(pos, true);
		refresh();
	}
}


//-----------------------------------------------------------------------
int NDKDlgListBox::get_cur_sel()
{
//	if(is_multisel())
//		throw NDKDlgException<NDKDlgListBox>(this,"Calling get_selected and is_multisel==true");
		
	return cur_sel;	
}


//-----------------------------------------------------------------------
void NDKDlgListBox::get_sel_items(IndexList &list)
{
	list.clear();
	if(is_multisel()){
		for(int i=0;i<data.size();i++){
			if(data[i]->is_checked())
				list.push_back(i);
		}
	}
	else
		list.push_back(cur_sel);
	
}


//-----------------------------------------------------------------------
void NDKDlgListBox::set_sel_items(const IndexList &list, bool t_or_f)
{
	if(is_multisel()){
		for(int i=0;i<list.size();i++){
			data[list[i]]->set_check(t_or_f);
		}
	}
	refresh_items();
	refresh();
}


//-----------------------------------------------------------------------
void NDKDlgListBox::check_multisel(int pos, bool t_or_f)
{
	if(!is_multisel()){
		if(t_or_f==true){
			cur_sel = pos;
			for(int i=0;i<data.size();i++){
				if(i != pos){
					data[i]->set_check(false);
					if(i>=top_item && i<=bottom_item){
						update_item(i);
					}
				}
			}
		}
		else //t_or_f == false
			cur_sel=-1;
	}
}


//-----------------------------------------------------------------------
void NDKDlgListBox::on_sel_change(int id)
{
	update_data(curr_item);
	check_multisel(curr_item,data[curr_item]->is_checked());
	refresh();
	if(notify_enabled())
		process_message(pParent,LI_SEL_CHANGE,get_ctrl_id(),0);
}	


/************************************************************************
*  	class NDKDlgSelectionBox											*
************************************************************************/
//-----------------------------------------------------------------------
void NDKDlgSelectionBox::initialize_items()
{
	curr_item=top_item=bottom_item=0;
	for(int i=pos_y;i<maxy();i++){
		items.push_back(new NDKDlgListItem(this,-1,"",1,width()-pos_x-has_frame() ,i,pos_x));
	}
	show_empty();
}


/************************************************************************
*  	class NDKDlgCheckListBox											*
************************************************************************/
//-----------------------------------------------------------------------
void NDKDlgCheckListBox::initialize_items()
{
	curr_item=top_item=bottom_item=0;
	for(int i=pos_y;i<maxy();i++){
		items.push_back(new NDKDlgCheckButton(this,-1,"",1,width()-pos_x-has_frame() ,i,pos_x));
	}
	show_empty();
}


/************************************************************************
*  	class NDKDlgChoiceBox												*
************************************************************************/
//-----------------------------------------------------------------------
void NDKDlgChoiceBox::initialize_items()
{
	curr_item=top_item=bottom_item=0;
	for(int i=pos_y;i<maxy();i++){
		items.push_back(new NDKDlgRadioButton(this,-1,"",1,width()-pos_x-has_frame() ,i,pos_x));
	}
	show_empty();
}


/************************************************************************
*  	class NDKDialogBox													*
************************************************************************/
//-----------------------------------------------------------------------
NDKDialogBox::NDKDialogBox(int id,const string &title
						,int lines, int cols,int y, int x,int style)
	: NDKDlgControl(id,lines,cols,y,x)
		,_title(title)
		,curr_child(0),
		dlg_style(style),
		changed(false),
		pBtnYes(0),
		pBtnNo(0),
		pBtnCancel(0),
		pBtnOk(0)
{
	leaveok(FALSE);	
	frame();
	show_text(_title,'c');
}

BEGIN_BINDINGS(NDKDialogBox,NDKDlgControl)
	ON_COMMAND(IDOK,NDKDialogBox::on_dlg_ok)
	ON_COMMAND(IDYES,NDKDialogBox::on_dlg_yes)
	ON_COMMAND(IDNO,NDKDialogBox::on_dlg_no)
	ON_COMMAND(IDCANCEL,NDKDialogBox::on_dlg_cancel)
END_BINDINGS()

//-----------------------------------------------------------------------
NDKDialogBox::~NDKDialogBox()
{ 
	hide();
	ListOfChildren::iterator it;
	for(it=children.begin();it!=children.end();it++){
		delete *it;
	}
}

		
//-----------------------------------------------------------------------
int NDKDialogBox::do_modal()
{
	on_dlg_init();
	bContinueModal=true;
	int req=REQ_DLG_EXIT;
	do{
		req=driver(getch());
		switch(req){
			case REQ_DLG_EXIT:	on_dlg_cancel();break;
			case REQ_DLG_NEXT:	next_control(); break;

			default:			on_cmd(req);	break;
		}	
	} while( bContinueModal);
	
	return user_request;
}


//-----------------------------------------------------------------------
void NDKDialogBox::show_text(const string &t,char where)
{
	if (!t.empty()){
		switch(where){
			case 'l': lefttext(0,t.c_str()); break;
			case 'c': centertext(0,t.c_str()); break;
			case 'r': righttext(0,t.c_str()); break;
			default: centertext(0,t.c_str()); break;
		}
	}
}


//-----------------------------------------------------------------------
int NDKDialogBox::driver (int key) 
{
	if(children[curr_child]->driver(key)==TRUE)
		return REQ_DLG_PROCESSED;
	switch(key) {
		case KEY_UP:	return REQ_DLG_UP;
		case KEY_DOWN:	return REQ_DLG_DOWN;
		case KEY_LEFT:	return REQ_DLG_LEFT;
		case KEY_RIGHT:	return REQ_DLG_RIGHT;
		case KEY_EXIT:
		case CTRL('X'):
		case KEY_ESC:
						///same as if user press Cancel button
						user_request=IDCANCEL; 
						return REQ_DLG_EXIT;
		case KEY_TAB:	return REQ_DLG_NEXT;
		default: return(key);
	}
}


//-----------------------------------------------------------------------
void NDKDialogBox::next_control()
{
	int start_point=curr_child;
	if(children.size()>1){
		for(;;){
			if(++curr_child==children.size())
				curr_child=0;
			if(curr_child==start_point)
				return;// We're back to the start_point.
			if(children[curr_child]->is_enabled())
				break; //We found the next selectable child
		}
		on_ctrl_term(*(children[start_point]));
		on_ctrl_init(*(children[curr_child]));
	}
}


//-----------------------------------------------------------------------
void NDKDialogBox::set_focus_to(int child_idx)
{
	on_ctrl_term(*(children[curr_child]));
	curr_child = child_idx;
	on_ctrl_init(*(children[curr_child]));
}


//-----------------------------------------------------------------------
void NDKDialogBox::set_focus_to(NDKDlgControl *p_child)
{
	for(int i=0;i<children.size();i++){
		if(children[i] == p_child){
			set_focus_to(i);
			break;
		}
	}
}


////-----------------------------------------------------------------------
//void NDKDialogBox::on_child_action(const NDKDlgControl *ctrl)
//{
//	user_request= ctrl->get_ctrl_id();
//	switch(user_request){
//		case IDOK:  on_dlg_ok(); break;	
//		case IDCANCEL: on_dlg_cancel(); break;	
//		case IDYES:	on_dlg_yes(); break;
//		case IDNO: on_dlg_no(); break;
//		default:break;
//	}
//	
//}


//-----------------------------------------------------------------------
bool NDKDialogBox::can_close()
{ 
	int result = true;
	if(changed){
		YesNoCancelBox dlg(titDataChgd,msgAskToSave);
		int usr_choice = dlg.do_modal();
		switch(usr_choice){
			case IDCANCEL: result = false;
			case IDYES: 
			case IDNO:
			default:
				break;
		}
	}
	refresh();
	return result;
}


//-----------------------------------------------------------------------
void NDKDialogBox::lefttext(int row,const char *label) {
  if (label) {
    OnError(addstr(row, 0, label, width()));
  }
}


//-----------------------------------------------------------------------
void NDKDialogBox::righttext(int row,const char *label) {
  if (label) {
    int x = maxx() - strlen(label);
    if (x<0)
      x=0;
    OnError(addstr(row, x, label, width()));
  }
}


//-----------------------------------------------------------------------
int NDKDialogBox::my_w = 0;		///useable width
int NDKDialogBox::btn_w= 0;		///each button width
int NDKDialogBox::btns=0;			///number of buttons
int NDKDialogBox::used_width=0;	///total buttons width			
int NDKDialogBox::spc=0;			///space between buttons
int NDKDialogBox::X_POS(int pos) 
{
	return (((my_w-used_width)/btns)+((btn_w+spc)*pos));
}


//-----------------------------------------------------------------------
void NDKDialogBox::create_buttons()
{
	switch((dlg_style & 0x00ff)){
		case okCancel:	add_ok_cancel(); 		break;
		case yesNo:		add_yes_no(); 			break;
		case yesNoCancel:add_yes_no_cancel(); 	break;
		default:		add_ok(); 				break;
	}
}

//-----------------------------------------------------------------------
void NDKDialogBox::add_ok()
{
	pBtnOk=new NDKDlgPushButton(this,IDOK,lblOk, maxy()-1,(maxx()-9)/2);
	add_child(pBtnOk);

	if(!children.empty())
		set_focus_to(children.size()-1);
}

//-----------------------------------------------------------------------
void NDKDialogBox::add_ok_cancel()
{
	NDKDlgControl *pCtrl=0;
	my_w = width()-2;	///useable width
	btn_w= 9;			///each button width
	btns=2;				///number of buttons
	used_width=btn_w*btns;	///total buttons width			
	spc=2;				///space between buttons
	pBtnOk=new NDKDlgPushButton(this,IDOK,lblOk,maxy()-1,X_POS(0));
	add_child(pBtnOk);
	pBtnOk->enable(false);
	pBtnCancel=new NDKDlgPushButton(this,IDCANCEL,lblCancel,maxy()-1,X_POS(1));
	add_child(pBtnCancel);
}


//-----------------------------------------------------------------------
void NDKDialogBox::add_yes_no_cancel()
{
	my_w = width()-2;	///useable width
	btn_w= 9;			///each button width
	btns=3;				///number of buttons
	used_width=btn_w*btns;	///total buttons width			
	spc=2;				///space between buttons
	pBtnYes=new NDKDlgPushButton(this,IDYES,lblYes,maxy()-1,X_POS(0));
	add_child(pBtnYes);
	pBtnNo=new NDKDlgPushButton(this,IDNO,lblNo,maxy()-1,X_POS(1));
	add_child(pBtnNo);
	pBtnCancel=new NDKDlgPushButton(this,IDCANCEL,lblCancel,maxy()-1,X_POS(2));
	add_child(pBtnCancel);
}


//-----------------------------------------------------------------------
void NDKDialogBox::add_yes_no()
{
	NDKDlgControl *pCtrl=0;
	my_w = width()-2;	///useable width
	btn_w= 9;			///each button width
	btns=2;				///number of buttons
	used_width=btn_w*btns;	///total buttons width			
	spc=2;				///space between buttons
	pBtnYes=new NDKDlgPushButton(this,IDYES,lblYes,maxy()-1,X_POS(0));
	add_child(pBtnYes);
	pBtnNo=new NDKDlgPushButton(this,IDNO,lblNo,maxy()-1,X_POS(1));
	add_child(pBtnNo);
}


/************************************************************************
*  	class NDKMessageBox													*
************************************************************************/
//-----------------------------------------------------------------------
NDKMessageBox::NDKMessageBox(	const string &title,const string &message,int style)
	: NDKDialogBox(10000,title,LINES-15,COLS-30,7,15,style),msg(message)
{
}


//-----------------------------------------------------------------------
void NDKMessageBox::on_dlg_init()
{	
	show_message();
	create_buttons(); 
	set_focus_to(pBtnCancel);
}


//-----------------------------------------------------------------------
void NDKMessageBox::show_message()
{
	int max_row_size = width()-4;
	int rows = msg.size()/max_row_size;
	if(msg.size()%max_row_size)
		rows++;
	
	for(int i=0;i<rows && i< maxy()-3;i++){
		string what(msg.substr(max_row_size*i,max_row_size));
		centertext(i+2,what.c_str());
	}
}


/************************************************************************
*  	class NDKApplication											*
************************************************************************/
//-----------------------------------------------------------------------
void NDKApplication::activateStatusBar()
{
	pStatusBar = new NDKStatusBar(1,COLS,LINES-1,0);
	
}


//-----------------------------------------------------------------------
bool NDKApplication::section_exists(ifstream &param_file,const string &section_name)
{
	string tag("[" + section_name + "]");
	string line;
	char line_read[255]={0};
	for(	param_file.getline(line_read,sizeof(line_read));
			param_file.good();
			param_file.getline(line_read,sizeof(line_read))){
		if(line_read[0] == '['){
			line = line_read;
			if(line.find(tag)!= string::npos){
				return true;
			}
		}
	}
	return false;
}


//-----------------------------------------------------------------------
long NDKApplication::find_key(ifstream &param_file,const string &section_name,
								const string &key_name,string &value)
{
	long line_beg =0;
	if(section_exists(param_file,section_name)){
		string line;
		string found_key;
		char line_read[255]={0};
		for(	param_file.getline(line_read,sizeof(line_read));
				param_file.good();
				param_file.getline(line_read,sizeof(line_read))){
			line = line_read;
			line_beg = param_file.tellg();
			line_beg -= line.size();
			if(line.empty() || line[0]=='#')///Skip empty and comment lines
				continue;
			if(line_read[0] == '[')///Reached the next section
				break;
			int from= line.find_first_not_of(" \t");
			if(from == string::npos)//Non-empty, but all spaces...
				continue;
			int to= line.rfind("#");///allow inline comments
			if(to!=string::npos){
				line = line.substr(from,to-from);
			}
			to= line.find_last_not_of(" \t");
			line = line.substr(from,to-from+1);
			int eq_sign = line.find('=');
			found_key= line.substr(0,eq_sign);
			if(found_key == key_name){
				if(to!=eq_sign && eq_sign!=string::npos){//no value...
					value= line.substr(eq_sign+1);
				}
				break;
			}
		}
	}
	return line_beg;
}


//-----------------------------------------------------------------------
string NDKApplication::get_param(const string &section_name,
								const string &key_name,const string &val)
{
	long pos=0;
	string value=val;
	if(!param_file_name.empty()){
		ifstream param_file(param_file_name.c_str());
		pos = find_key(param_file,section_name,key_name,value);
		if(pos && value.empty())
			value=val;
	}
		
	return value;	
}


//-----------------------------------------------------------------------
long NDKApplication::get_param(const string &section_name,
								const string &key_name,long val)
{
	char txt[255]={0};
	ostrstream str_val(txt,sizeof(txt));
	str_val << val << ends;
	string result = get_param(section_name,key_name, str_val.str());
	return atol(result.c_str());
;
}

} //namespace NDK_XX
