// * This makes emacs happy -*-Mode: C++;-*-
/****************************************************************************
 * Copyright (c) 2002 Fabrica de Software XXI SRL - info@lafabrica.com.ar	*
 * Copyright (c) 2002 RaqLink SA - info@raqlink.com							*
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 *	Author Miguel Pinkas <miguel@lafabrica.com.ar> 							*
 ****************************************************************************/
#if !defined(__NDK_NCURSES__)
#define __NDK_NCURSES__
#include <cursesapp.h>
#include <cursesm.h>
#include <cursesf.h>
#include <vector>
#include <string>
#include <fstream>
#include <typeinfo>
using namespace std;

#if HAVE_LIBC_H
#  include <libc.h>
#endif

#include "dlgdefines.h"

namespace NDK_XX {

ofstream &get_logfile();

/************************************************************************
*  	class NDKMenuItem													*
* 	A derivative of NCursesMenuItem to add some nice stuff				*
*					 													*
************************************************************************/
class NDKMenuItem : public NCursesMenuItem
{
  friend class NDKMenu;
public:
	NDKMenuItem(const char *p_name= 0,const char *p_descript=0 )
		:NCursesMenuItem(	p_name ,p_descript){
		
	};
	
	///Access to the underlying item;
	ITEM *get_curses_item()const{return item;}
	
	///Access to our parent C++ menu object
	NDKMenu *get_menu() const { return pMenu;}
protected:

	///The C++ menu object we belong to
	NDKMenu *pMenu;

};

typedef vector<NCursesMenuItem *> ListOfItems;


/************************************************************************
*  	class NDKMenu														*
* 	A derivative of NCursesMenu to add some nice stuff					*
*					 													*
************************************************************************/
class NDKMenu : public NCursesMenu
{
public:
	// Make a menu with a window of this size.
	NDKMenu (ListOfItems &items, 
		int  lines, 
		int  cols, 
		int  begin_y = 0, 
		int  begin_x = 0,
		bool framed=FALSE,        
		bool own_items=FALSE);  // Autocleanup of Items?

	NDKMenu( int  lines, int  cols, int  begin_y = 0, int  begin_x = 0
								,bool framed=FALSE); 


	///We use the next three to autosize menubars
	int get_itemwidth() const	{ return menu? menu->itemlen + menu->spc_cols:0;} 
	int get_itemmarklen() const	{ return menu? menu->marklen:0;} 
	int get_spcrows() const		{ return menu? menu->spc_rows:0;} 

	
	// Do that silly traversal with a real iterator...
	static short count_items(NCursesMenuItem* nitems[]) {
		int itemCount = 0;
		for (int lcv=0; nitems && nitems[lcv]->name(); ++lcv){
			++itemCount;
		}
		return itemCount;
	}
	
	///Hide good ol' pointers from here on...
	void setItems(ListOfItems &items){NCursesMenu::setItems(items.begin());	}

	///Hide good ol' pointers from here on...
	void InitMenu(ListOfItems &items){
		NCursesMenu::InitMenu(items.begin(),is_framed(),FALSE);
	}

	/// Put the label text at the left 
	/// (0) in the specified row.
	virtual void lefttext(int row,const char* label);

	/// Put the label text to the right 
	/// (ending at maxx()) in the specified row.
	virtual void righttext(int row,const char* label);
	
	///Unfortunately, b_framed on parent 
	///is private, so we can't check it...:-(
	bool is_framed() const {return have_frame;}
	
protected:

	///Unfortunately, b_framed on parent 
	///is private, so we can't check it...:-(
	bool have_frame;
	
	///We added a link to this C++ menu object 
	///to our c++ item object.
	///Here we loop through the items linking them to us
	void link_items(ListOfItems &items);
};


/************************************************************************
*  	class NDKMenuBar													*
* 	A menu bar is a menu containing popup and regular items				*
*   by default, it has centered top and bottom labels and 				*
*	wide orientation 													*
************************************************************************/
class NDKMenuBar : public NDKMenu 
{
protected:
	char orientation;
	string title;
	string bTitle;
	string titlePos;
	ListOfItems &itemList;
	
	/// A simple helper
	void show_text(const string &t,char where);
	
public:
	NDKMenuBar (int rows, int cols,int y, int x
  					,ListOfItems &items
					,const char *ttitle="", const char *btitle=""
					,const char *title_pos= "cc"
					,char orient='h',bool framed=false);

	~NDKMenuBar(){ }	


	void setTitle(){
		show_text(title,titlePos[0]);
		show_text(bTitle,titlePos[1]);
	}
	
	///we are wide-oriented or tall-oriented?
	char get_orientation()const{return orientation;}
};


/************************************************************************
*  	class NDKFramedMenuBar												*
* 	Ok,don't need to explain what is this, right...?					*
************************************************************************/
class NDKFramedMenuBar : public NDKMenuBar
{
public:
	NDKFramedMenuBar (int rows, int cols,int y, int x
  					,ListOfItems &items
					,const char *ttitle="", const char *btitle=""
					,const char *title_pos= "cc"
					,char orient='h')
		:  NDKMenuBar (rows+2,cols,y,x,items,ttitle,btitle,title_pos
						,orient,true)
  	{
  	}
};


/************************************************************************
*  	class NDKSubMenu													*
* 	A Menu that delpoys itself relative to a popup menu item.			*
************************************************************************/
class NDKSubMenu : public NDKMenuBar
{
protected:
public:
	NDKSubMenu (	int rows, int cols,int y, int x,
  					ListOfItems &items,
					const char *ttitle="", const char *btitle="", 
					const char *title_pos= "l", 
					char orient='v',bool framed=true)
		:  NDKMenuBar (rows+2, cols+3,y,x,items,ttitle,btitle,title_pos
						,orient,framed)
  	{
  	}
	~NDKSubMenu() { 
	}
	
	void setItems(ListOfItems &items);
};


/************************************************************************
*  	class NDKPopUpMenu													*
* 	A popup menu item which displays a submenu.							*
*	the popup menu manages the liftime of the items.					*
************************************************************************/
class NDKPopUpMenu : public NDKMenuItem
{
public:
	NDKPopUpMenu(const char *text=0) : NDKMenuItem(text){	}
	~NDKPopUpMenu();
	
	bool action();
	
	///Autocalculated starting column for the submenu
	short submenux() const;
	
	///Autocalculated starting row for the submenu
	short submenuy()const;
		
	/// So we can know the right width for the submenu
	int largest_name_len()const;

protected:
	///These are passed to the submenu...
	ListOfItems items;

	/// Since submenu is autosized, we can pass
	/// longer (i.e. meaningful) labels to it...
	string sub_menu_label;
};


struct BINDING;
struct BINDING_MAP;

/************************************************************************
*  	class NDKDlgControl													*
* 	A Panel which behaves as a basic dialog control.					*
*	Is the base class for all the dialog controls						*
************************************************************************/
class NDKDlgControl : public NCursesPanel
{
	friend class NDKDialogBox;
public:
	///The styles a control can have
	enum styles{
		framed  = 	0x0001,	///Do we have a frame?
		moveable=   0x0002,	///We can be moved around?
		editable=   0x0004,	///Is our text editable?
		tabstop =	0x0008,	///Can be looped thru?
		notify	=	0x0010,	///Do we notify our parent of events?
		hscroll =	0x0020, ///Can we scroll horizontally?
		multisel=	0x0040,	///Can select multiple items?
		hscrollbar= 0x0100, ///Draw horizontal scrollbar?
		vscrollbar= 0x0200  ///Draw vertical scrollbar?
	};
	
	///The states a control can take
	enum states{
		unknown =	0x0000,
		active	= 	0x0001,	///aka Focused
		ovrwrt	=	0x0002,	///if editable==true, insert or overwrite?
		checked= 	0x0004,	///If a selectable control, are we checked?
		visible =	0x0008	///dito
	};

	NDKDlgControl(	NDKDlgControl *parent, int id
						,int lines,int cols,int y, int x,int style=0)
    	: NCursesPanel(lines,cols,parent->begy()+y,parent->begx()+x)
			,pParent(parent),ctrl_id(id)
			,field_style(style),field_state(unknown)
	{
		if(has_frame()){
			box();
		}
		set_visible();
  	}
	
	NDKDlgControl(int id
						,int lines,int cols,int y, int x,int style=0)
    	: NCursesPanel(lines,cols,y,x)
			,pParent(0),ctrl_id(id)
			,field_style(style),field_state(unknown)
	{
		if(has_frame()){
			box();
		}
		set_visible();
  	}
	///tell me what styles this window has...
	styles get_style() const { return styles(field_style); }
	 
	///	set all styles at once...
	styles set_styles(styles s);
	
	/// set/unset a style bit
  	styles set_style(styles s,bool on=true);
	

	///tell me what status this window is on...
	states get_state() const { return states(field_state); }
	
	///	set all states at once...
	states set_states(states s);
	
	/// set/unset a state bit
  	states set_state(states s,bool on=true);
	
	/// if this style has been set...
	bool has_frame()const{return ( (field_style & framed)== framed);}
	

	/// the user can move this control?...
	bool is_moveable() const{ return ( (field_style & moveable)==moveable);}
	
	/// the user can edit this control's text?
	bool is_editable() const{ return ( (field_style & editable)==editable);}
	
	/// are we visible on the list viewport?
	bool is_visible() const{ return ( (field_state & visible)==visible);}
	
	/// a control always moves inside its parent boundaries,
	/// so make this function use only relative values...
	int mvwin(int y, int x);
	
	
	/// Usualy, a control in a dialog has some kind of text on it...
	virtual void set_text(const string &txt)=0;
	virtual string get_text()const=0 ;
	
	/// The control identifier
	int get_ctrl_id() const {return ctrl_id;};
	void set_ctrl_id(int id){ctrl_id=id;};

	/// Maybe these should change to set_focus/unset_focus
	virtual void activate();
	virtual void deactivate();
	
	///show or hide
	void set_visible(bool new_state=true){set_state(visible,new_state);}
	
	///Ignore /Stop ignoring us when user press tab
	void enable(bool t_or_f=true){ set_style(tabstop,t_or_f);}

	/// can make this control the active control?
	bool is_enabled()const{ return ( (field_style & tabstop)==tabstop);}
	
	///Ignore /Stop ignoring us when user press tab
	void set_hscroll(bool t_or_f=true){ set_style(hscroll,t_or_f);}

	/// can make this control the active control?
	bool is_hscroll_set()const{ return ( (field_style & hscroll)==hscroll);}

	///Allow multiple selections
	void set_multisel(bool t_or_f=true){ set_style(multisel,t_or_f);}

	/// can make this control the active control?
	bool is_multisel()const{ return ( (field_style & multisel)==multisel);}

	///notify parent of changes
	void enable_notify(bool t_or_f=true){ set_style(notify,t_or_f);}

	/// should we notify our parent upon firing of action()?
	bool notify_enabled() const{ return ( (field_style & notify)==notify);}
	
	///Set edit mode to overwrite or insert
	void set_overwrite(bool t_or_f=true){ set_state(ovrwrt,t_or_f);}
	
	/// the user current pos is overwritten or char is inserted?
	bool overwrite()const
		{ return (is_editable() &&( (field_state & ovrwrt)==ovrwrt));}
		

	/// if this style has been set...
	bool has_hscrollbar()const{return ( (field_style & hscrollbar)== hscrollbar);}
	void set_hscrollbar(bool t_or_f=true){ set_style(hscrollbar,t_or_f);}
	
	bool has_vscrollbar()const{return ( (field_style & vscrollbar)== vscrollbar);}
	void set_vscrollbar(bool t_or_f=true){ set_style(vscrollbar,t_or_f);}
				
	void show(){
		if(is_visible()) return;
		NCursesPanel::show();
		set_visible();
		enable();
	}
	
	void hide(){
		if(!is_visible()) return;
		NCursesPanel::hide();
		set_visible(false);
		enable(false);
	}

			
	/// Delegation function to get the params from almost anywhere in NDK++
	string get_param(const string &section_name
							,const string &key_name
							,const string &val);

	/// Delegation function to get the params from almost anywhere in NDK++
	long get_param(const string &section_name
							,const string &key_name
							,long val);

	/// Delegation function to save the params from almost anywhere in NDK++
	string save_param(const string &section_name
							,const string &key_name
							,const string &val);

	/// Delegation function to save the params from almost anywhere in NDK++
	long save_param(const string &section_name
							,const string &key_name
							,long val);
	static ofstream &send_to_log(const string &what);

protected:
	NDKDlgControl *pParent;
	int ctrl_id;
	int field_style;
	int field_state;
	
	///we won't notify parent if true
	bool on_initial_update; 
	
	/// get a chance at processing the key,
	/// before parent does
	virtual int driver(int key){ return FALSE;}
	
private: 
	static const BINDING _bindingEntries[]; 
protected: 
	static const BINDING_MAP bindingMap; 
	virtual const BINDING_MAP* get_binding_map() const; 
//	static bool resolve_entry(int ctrl_id, int what,void (NDKDlgControl::*)(int)pfn, void* pExtra);
	static bool process_message(NDKDlgControl *pTarget
									, int what, int ctrl_id, void* pExtra);
	static const BINDING* get_binding(const BINDING* pEntry
											,int what, int ctrl_id);
};

#include "ndk_bindings.h"

/************************************************************************
*  	class NDKDlgLabel													*
* 	A static label  la M$.												*
************************************************************************/
class NDKDlgLabel : public NDKDlgControl
{
public:
	NDKDlgLabel( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify);

	NDKDlgLabel( int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify);
	~NDKDlgLabel() {}

	void set_text(const string &txt);
	string get_text() const {return text;}
	
	void activate();
	void deactivate();

	virtual void clear_text(bool ref=false);
	
	///Useful for scrolling labels in lists
	chtype operator[](int pos)const
	{
		chtype result;
		
		if(pos < 0 || pos >= text.size())
			result = getbkgd();
		else 
			result = text[pos];
		
		return result;
	}
	int visible_size() const {return (text.size()-x_start);}

	int max_len() const {return viewport->width();}
protected:
	int driver(int key);
	
	string text;
	
	///Hack to manage framed/unframed windows
	int pos_x, pos_y;

		
	///Edition viewport.
	///When not framed, viewport = window;
	NCursesWindow *viewport;
	
	///cursor situation
	int cur_x, cur_y;
	
	/// first visible char
	int x_start;
};


/************************************************************************
*  	class NDKDlgEdit													*
* 	A single line, editable control  la M$.							*
************************************************************************/
class NDKDlgEdit : public NDKDlgLabel
{
public:
	NDKDlgEdit( NDKDlgControl *parent,int id,const string &text
					,int lines,int cols,int y, int x
					,int style=framed|editable|tabstop|notify);

	~NDKDlgEdit();
	int edit(int key);
	
protected:
	virtual int add_or_insert_char(int key);
	virtual void add_char(int key);
	virtual void insert_char(int key);

	int driver(int key);
};


/************************************************************************
*  	class NDKDlgButton													*
*	The base class for radio buttons, push buttons and check boxes		*
************************************************************************/
class NDKDlgButton : public NDKDlgLabel
{
public:
	NDKDlgButton( NDKDlgControl *parent,int id, const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify);

	void activate();
	void deactivate();

	virtual int action()=0;
	void set_default(bool t_or_f=true) {default_choice=t_or_f;}
	bool is_default()const {return default_choice;}
	
protected:
	int driver(int key);
	bool default_choice;
};


/************************************************************************
*  	class NDKDlgPushButton												*
************************************************************************/
class NDKDlgPushButton : public NDKDlgButton
{
public:
	NDKDlgPushButton( NDKDlgControl *parent,int id, const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify|framed);

	NDKDlgPushButton( NDKDlgControl *parent,int id, const string &title
					,int y, int x);
					
	void set_text(const string &txt);

	int action(){ 
		process_message(pParent,0,get_ctrl_id(),0);
		return TRUE;
	}
};


/************************************************************************
*  	class NDKDlgListItemData											*
************************************************************************/
class NDKDlgListItem; //forward reference
class NDKDlgListItemData
{
public:
	NDKDlgListItemData(	const string &txt,int id=-1
						,int style=NDKDlgControl::tabstop|NDKDlgControl::notify
						,int state=0,void *data=0)
		:text(txt),ctrl_id(id),field_style(style)
			,field_state(state),user_data(data)
		{
		}

	NDKDlgListItemData &operator=(const NDKDlgListItem &lha);
	NDKDlgListItemData &operator=(const NDKDlgListItemData &lha);
	
	void set_text(const string &txt){text = txt;}
	string get_text()const 	{return text;}


	void set_ctrl_id(int id)		{ctrl_id = id;}
	int get_ctrl_id()	const	{return ctrl_id;}

	void set_styles(int style)	{field_style = style;}
	int get_style()	const 		{return field_style;}

	void set_states(int state)	{field_state = state;}
	int get_state()	const 		{return field_state;}

	/// set/unset a state bit
  	int set_state(int s,bool on=true);

	void set_user_data(void *data)	{user_data = data;}
	void *get_user_data() const 	{return user_data;}

	void set_check(bool new_state=true){
		bool prev_state=is_checked();
		set_state((int)NDKDlgControl::checked,new_state); 
	};
	
	/// is this item selected?
	bool is_checked() const 
	{ return ( (field_state & NDKDlgControl::checked)
					== NDKDlgControl::checked);}

	void set_default(bool t_or_f=true) {default_choice=t_or_f;}
	bool is_default()const {return default_choice;}
	
protected:
	string text;
	int ctrl_id;
	int field_style;
	int field_state;
	bool default_choice;
	void *user_data;
};


/************************************************************************
*  	class NDKDlgListItem												*
************************************************************************/
class NDKDlgListItem : public NDKDlgButton
{
	friend class NDKDlgListBox;
public:
	NDKDlgListItem( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x,int style=tabstop|notify);

	void set_text(const string &txt);

	///select/unselect item
	void set_check(bool new_state=true){
		bool prev_state=is_checked();
		set_state(checked,new_state); 
		show_state(true);
		if(prev_state!=new_state)
			process_message(pParent,LI_SEL_CHANGE,get_ctrl_id(),0);
	};
	
	/// is this item selected?
	bool is_checked() const { return ( (field_state & checked)==checked);}
	
	void set_user_data(void *data){user_data = data;}
	void *get_user_data() const {return user_data;}
	
	NDKDlgListItem &operator=(const NDKDlgListItemData &lha);
	
protected:
	void *user_data;

	int driver(int key);

	///specialized on children
	virtual void show_state(bool ref=false)
	{
		if(!text.empty()){
			if(is_checked()) addch(0,0, '*');
			else addch(0,0, ' ');
		}
		if(ref)refresh();
	}
	
	///specialized on children
	virtual int state_size() const {return 1;};

	///Hidden from public use
	int action() {return FALSE;}

	///parent notification
//	void notify_change_to_parent()const;
//	void notify_action_to_parent()const;

};




/************************************************************************
*  	class NDKDlgRadioButton												*
************************************************************************/
class NDKDlgRadioButton : public NDKDlgListItem
{
public:
	NDKDlgRadioButton( NDKDlgControl *parent,int id, const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify);

	NDKDlgRadioButton( NDKDlgControl *parent,int id, const string &title
					,int y, int x);

protected:

	void show_state(bool ref=false){
		if(!text.empty()){
			if(is_checked()) addstr(0,0, "(0)");
			else addstr(0,0, "( )");
		}
		if(ref)refresh();
	}
	
	int state_size() const {return 3;};
};


/************************************************************************
*  	class NDKDlgCheckButton												*
************************************************************************/
class NDKDlgCheckButton : public NDKDlgListItem
{
public:
	NDKDlgCheckButton( NDKDlgControl *parent,int id, const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|notify);

	NDKDlgCheckButton( NDKDlgControl *parent,int id, const string &title
					,int y, int x);

protected:

	void show_state(bool ref=false){
		if(!text.empty()){
			if(is_checked()) addstr(0,0, "[x]");
			else addstr(0,0, "[ ]");
		}
		if(ref)refresh();
	}
	
	int state_size() const {return 3;};
};



/************************************************************************
*  	class NDKDlgListBox													*
*	Base class for managging lists of things							*
************************************************************************/
typedef vector<NDKDlgListItem *> VisibleItems;
typedef vector<NDKDlgListItemData *> ListItems;
typedef vector<int> IndexList;

class NDKDlgListBox : public NDKDlgControl
{
	friend class NDKDlgListItem;
public:
	NDKDlgListBox( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|framed|notify);
	
	~NDKDlgListBox();
	
	///You can add an item just sending in a string
	int add_item(const string &new_item, int pos=-1,bool is_default= false,bool make_visible=true);
	
	///	Here you are sending a full item data ptr.
	///	The list will own it and dispose it at destruction time
	int add_item(NDKDlgListItemData *new_item,int pos=-1,bool make_visible=true);
	
	/// Drop an item from the list
	int remove_item(int pos=-1);
		
	///Here we will manage the virtualized list
	int refresh();
	
	void activate();
	void deactivate();
	
	string get_text() const {return _title;}
	void set_text(const string &v){
		_title = v;
		show_text(v,'l');
	}
	
	virtual int get_cur_pos();
	virtual void set_cur_pos(int pos=-1);

	virtual int get_cur_sel();
	void set_cur_sel(int pos);
	
	virtual void get_sel_items(IndexList &list);
	virtual void set_sel_items(const IndexList &list, bool t_or_f=true);
	
	virtual string get_item_text(int pos=-1);
	virtual void set_item_text(int pos,const string &txt);
		
	virtual void *get_item_data(int pos=-1);
	virtual void set_item_data(int pos,void *data);

	virtual NDKDlgListItemData *get_item(int pos=-1);
	virtual void set_item(int pos,NDKDlgListItemData *item);

	virtual int get_count(){return data.size();};

	void clear();

protected:
	string _title;
	int pos_x, pos_y;

	VisibleItems items;
	ListItems data;
	
	int curr_item;
	int top_item;
	int bottom_item;
	int cur_sel;
	int driver(int key);

	/// Put the label text at the left (0) 
	/// in the specified row.
	virtual void lefttext(int row,const char* label);

	/// Put the label text to the right (ending at maxx())
	/// in the specified row.
	virtual void righttext(int row,const char* label);

	void show_text(const string &t,char where);

	virtual void adjust_top_and_bottom();

	virtual void move_up();	
	virtual void move_down();
	virtual void move_left();
	virtual void move_right();
	
	virtual void activate_item(int item_idx);
	virtual void deactivate_item(int item_idx);

	void update_data(int item_idx);
	void update_item(int item_idx);
	
	void refresh_items();
	void clear_items();
	void delete_data();
	void show_empty();
	
	void draw_scrollbar();
	NDKDlgListItem *&visible_item(int requested)
									{return items[requested-top_item];}

	void check_multisel(int pos, bool t_or_f);
	
	void on_sel_change(int);
	
	DECLARE_BINDINGS();
};


/************************************************************************
*  	class NDKDlgSelectionBox											*
*																		*
************************************************************************/
class NDKDlgSelectionBox : public NDKDlgListBox
{
public:
	NDKDlgSelectionBox( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|framed|notify)
		:NDKDlgListBox( parent,id,title
					,lines,cols,y, x
					,style)
		{
			initialize_items();
		}
					
protected:
	void initialize_items();
};


/************************************************************************
*  	class NDKDlgCheckListBox											*
*																		*
************************************************************************/
class NDKDlgCheckListBox : public NDKDlgListBox
{
public:
	NDKDlgCheckListBox( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|framed|notify)
		:NDKDlgListBox( parent,id,title
					,lines,cols,y, x
					,style)
		{
			initialize_items();
		}
					
protected:
	void initialize_items();
};


/************************************************************************
*  	class NDKDlgChoiceBox												*
*																		*
************************************************************************/
class NDKDlgChoiceBox : public NDKDlgListBox
{
public:
	NDKDlgChoiceBox( NDKDlgControl *parent,int id,const string &title
					,int lines,int cols,int y, int x
					,int style=tabstop|framed|notify)
		:NDKDlgListBox( parent,id,title
					,lines,cols,y, x
					,style)
		{
			initialize_items();
		}
					
protected:
	void initialize_items();
};


/************************************************************************
*  	class NDKDialogBox													*
* 	A dialog box made up from a framed pad and a panel.					*
************************************************************************/
typedef vector <NDKDlgControl *> ListOfChildren;

class NDKDialogBox: public NDKDlgControl
{
public:
	enum dlg_styles{
		info		= 0x0100,
		warning		= 0x0200,
		error		= 0x0400,
		okCancel	= 0x0001,
		yesNo		= 0x0002,
		yesNoCancel	= 0x0004
	};

	enum Dlg_Request{
		DLG_LOW = PAD_HIGH + 1,
		REQ_DLG_REFRESH = DLG_LOW,
		REQ_DLG_PROCESSED,
		REQ_DLG_NEXT,
		REQ_DLG_PREV,
		REQ_DLG_UP,
		REQ_DLG_DOWN,
		REQ_DLG_LEFT,
		REQ_DLG_RIGHT,
		REQ_DLG_EXIT,
		DLG_HIGH = REQ_DLG_EXIT
	};

	NDKDialogBox(int id, const string &title
						,int lines, int cols,int y=0, int x=0
						,int style=0);
	~NDKDialogBox();
	
	void add_child(NDKDlgControl *new_child){ 
			children.push_back(new_child);
	}	
	
	string get_text() const {return _title;}
	void set_text(const string &v){
		_title = v;
		show_text(v,'c');
	}

	int do_modal();

	static NCursesPanel dlg_stdscr;
	
protected:
	///Allow children to touch our stuff
	friend NDKDlgControl;
	/// The controls we have
	ListOfChildren children;
	
	/// where is te focus now
	int curr_child;
	
	string _title;
	
	int dlg_style;
	
	///Modal loop condition
	bool bContinueModal;
	
	///So we can ask the user if she want to save
	bool changed;
	
	NDKDlgPushButton *pBtnYes;
	NDKDlgPushButton *pBtnNo;
	NDKDlgPushButton *pBtnCancel;
	NDKDlgPushButton *pBtnOk;

	///this is the result of the interaction
	int user_request;
	
	/// A simple helper
	void show_text(const string &t,char where);

	/// When user press TAB, give me the next child
	void next_control();

protected:
	DECLARE_BINDINGS();

	///called at the beggining of the modal loop
	virtual void on_dlg_init(){}
		
	///called at the end of the modal loop
	virtual void on_dlg_term(){	}
	
	///called if user presses one of the 'exits' 
	virtual bool can_close();
	
	///called when the user enters a control
	virtual void on_ctrl_init(NDKDlgControl &ctrl){
		ctrl.activate();
	}
	
	///called when the user leaves the control
	virtual void on_ctrl_term(NDKDlgControl &ctrl){
		ctrl.deactivate();
	}
	
	/// Kind of catch-all for derived windows
	virtual void on_cmd(int dlg_req) {
	}
	
	///called on construction of dialog
	virtual void create_buttons();
	
	///The next four are used by create_buttons
	void add_ok();
	void add_ok_cancel();
	void add_yes_no(); 
	void add_yes_no_cancel(); 

	///activate child at position child_idx
	virtual void set_focus_to(int child_idx);

	///activate p_child
	virtual void set_focus_to(NDKDlgControl *p_child);

	int driver (int key);
	
	///Vanilla implementation of exits
	virtual void on_dlg_ok()	{end_dialog(IDOK);} 
	virtual void on_dlg_yes()	{end_dialog(IDYES);}
	virtual void on_dlg_no()	{end_dialog(IDNO);}
	virtual void on_dlg_cancel(){
		if(can_close())
			end_dialog(IDCANCEL);
	}

	///Break the modal loop
	///and give the user a chance to cleanup/serialize/whatever
	int end_dialog(int request){
		user_request = request;
		bContinueModal=false;
		on_dlg_term();
	}

	/// Put the label text at the left (0) 
	/// in the specified row.
	virtual void lefttext(int row,const char* label);

	/// Put the label text to the right (ending at maxx())
	/// in the specified row.
	virtual void righttext(int row,const char* label);
	static int my_w;		///useable width
	static int btn_w;		///each button width
	static int btns;			///number of buttons
	static int used_width;	///total buttons width			
	static int spc;			///space between buttons
	static int X_POS(int pos);

	static string lblOk;
	static string lblCancel;
	static string lblYes;	
	static string lblNo;
	static string titDataChgd;
	static string msgAskToSave;
};


/************************************************************************
*  	class NDKMessageBox													*
* 	You already know what is this... ;-)								*
************************************************************************/
class NDKMessageBox: public NDKDialogBox 
{
public:
	NDKMessageBox(const string &title,const string &message,int style=info);
protected:

	///Prompt the user
	string msg;

	///We redefine here so we get called on construction
	void on_dlg_init(); 
	void show_message();
};

typedef NDKMessageBox  MessageBox;


/************************************************************************
*  	class NDKOkCancelBox													*
* 	You already know what is this... ;-)								*
************************************************************************/
class NDKOkCancelBox: public NDKMessageBox 
{
public:
	NDKOkCancelBox(const string &title,const string &message,int style=info)
		:NDKMessageBox(title,message,style|okCancel)
		{};
};

typedef NDKOkCancelBox  OkCancelBox;


/************************************************************************
*  	class NDKYesNoBox													*
* 	You already know what is this... ;-)								*
************************************************************************/
class NDKYesNoBox: public NDKMessageBox 
{
public:
	NDKYesNoBox(const string &title,const string &message,int style=info)
		:NDKMessageBox(title,message,style|yesNo)
		{};
};

typedef NDKYesNoBox  YesNoBox;


/************************************************************************
*  	class NDKYesNoBox													*
* 	You already know what is this... ;-)								*
************************************************************************/
class NDKYesNoCancelBox: public NDKMessageBox 
{
public:
	NDKYesNoCancelBox(const string &title,const string &message,int style=info)
		:NDKMessageBox(title,message,style|yesNoCancel)
		{};
};
typedef NDKYesNoCancelBox  YesNoCancelBox;


/************************************************************************
*  	class NDKStatusBar													*
*	No need to explain this, right...? :-)								*
************************************************************************/
class NDKStatusBar : public NDKDlgLabel
{
public:

	NDKStatusBar(int lines,int cols,int y, int x,const string &text="Status bar")
    	: NDKDlgLabel(1000,text
						,lines, cols, y, x, visible)
	{
  	}
};


/************************************************************************
*  	class NDKApplication												*
*	Derivation of the NCursesApplication class							*
*	where we added a status bar											*
************************************************************************/
class NDKApplication: public NCursesApplication
{
protected:

	NDKApplication(bool wantColors = FALSE)
  		:	NCursesApplication(wantColors),pStatusBar(0)
	{
  	}
	
	virtual void activateMenuBar(ListOfItems &items)=0;
	virtual void activateStatusBar();
  
public:

	static NDKApplication *get_application(){
		NDKApplication *pApp = dynamic_cast<NDKApplication *>
										(NCursesApplication::getApplication());
		return pApp;
	}
	
	virtual ~NDKApplication(){
  		delete pStatusBar;
	};

	NDKStatusBar *getStatusBar() {return pStatusBar;}

	///Tell us what is the name of the parameters file	
	void set_param_file_name(const string &file_name){param_file_name = file_name;}
	
	/// Retrive the parameters file name
	string get_param_file_name()const {return param_file_name;}
	
	/// Find a string parameter (a la MS's .ini files)
	string get_param(const string &section_name
						,const string &key_name
						,const string &val);

	/// Find a numeric parameter (a la MS's .ini files)
	long get_param(const string &section_name
						,const string &key_name
						,long val);

	/// Save a string parameter (a la MS's .ini files)
	string save_param(const string &section_name
						,const string &key_name
						,const string &val){};

	/// Save a numeric parameter (a la MS's .ini files)
	long save_param(const string &section_name
						,const string &key_name
						,long val){};

protected:

	NDKStatusBar *pStatusBar;
	string param_file_name;
	
	///Helper to check if a section given in ge/save_param functs. exists
	bool section_exists(ifstream &param_file,const string &section_name);
	
	///Helper to check if a key given in ge/save_param functs. exists
	long find_key(ifstream &param_file,const string &section_name
							,const string &key_name,string &value);

};


/// Delegation function to get the params from almost anywhere in NDK++
inline long NDKDlgControl::get_param(const string &section_name
									,const string &key_name
									,long val)
{
	return	NDKApplication::get_application()->get_param(section_name,key_name,val);
}


/// Delegation function to get the params from almost anywhere in NDK++
inline string NDKDlgControl::get_param(const string &section_name
									,const string &key_name
									,const string &val)
{
	return	NDKApplication::get_application()->get_param(section_name,key_name,val);
}


/// Delegation function to set the params from almost anywhere in NDK++
inline long NDKDlgControl::save_param(const string &section_name
									,const string &key_name
									,long val)
{
	return	NDKApplication::get_application()->save_param(section_name,key_name,val);
}


/// Delegation function to set the params from almost anywhere in NDK++
inline string NDKDlgControl::save_param(const string &section_name
									,const string &key_name
									,const string &val)
{
	return	NDKApplication::get_application()->save_param(section_name,key_name,val);
}

/************************************************************************
*  	class ClosingMarkItem												*
* 	The end marker for an items list.									*
************************************************************************/
class ClosingMarkItem : public NDKMenuItem
{
protected:
public:
  ClosingMarkItem() : NDKMenuItem() { }
  ~ClosingMarkItem(){ 
  }
};


/************************************************************************
*  	class NonImplementedItem											*
* 	A non-functional item (you can draft your menu with these.			*
************************************************************************/
class NonImplementedItem : public NDKMenuItem {
public:
  	NonImplementedItem(const char *text) : NDKMenuItem(text) {
    	options_off(O_SELECTABLE);
  	}
  	~NonImplementedItem(){
	}
};


/************************************************************************
*  	class ExitCmd														*
* 	This is the one to take us off the menu.							*
************************************************************************/
class ExitCmd : public NDKMenuItem
{
public:
  ExitCmd(const char *text="Exit") : NDKMenuItem(text) { }
  ~ExitCmd(){}
  bool action() { return TRUE; }
};


/************************************************************************
*  	class BackCmd														*
* 	Useful if you don't have a mouse.									*
*	Put it first on your drop down menu so you are 						*
*	just an enter away from the menu bar								*
************************************************************************/
class BackCmd : public NDKMenuItem
{
public:
  BackCmd(const char *text="Back") : NDKMenuItem(text) { }
  ~BackCmd(){}
  bool action() { return TRUE; }
};



/************************************************************************
*  	template class NDKDlgException										*
*	We use it from anywhere in NDK++ to throw using the 				*
*	current context (usually the window containing the offending code)	*
************************************************************************/
template<class T> class NDKDlgException : public NCursesException
{
public:
  const T* pElem;

  NDKDlgException (const char *msg, int err) : 
    NCursesException (msg, err),
    pElem (0)
    {};

  NDKDlgException (const T* elem,
			 const string &msg,
			 int err=-1) : 
    NCursesException (msg.c_str(), err),
    pElem (elem)
    {};

  NDKDlgException (int err) : 
    NCursesException ("NDK++ library exception", err),
    pElem (0)
    {};

  NDKDlgException (const T* elem,
			 int err) : 
    NCursesException ("NDK++ library exception", err),
    pElem (elem)
    {};

  virtual const char *classname() const {
    return typeid(pElem).name();
  }

};



} //namespace NDK_XX




#endif //!defined(__NDK_NCURSES__)
