// * This makes emacs happy -*-Mode: C++;-*-
/****************************************************************************
 * Copyright (c) 2002 Fabrica de Software XXI SRL - info@lafabrica.com.ar	*
 * Copyright (c) 2002 RaqLink SA - info@raqlink.com							*
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 *	Author Miguel Pinkas <miguel@lafabrica.com.ar> 							*
 ****************************************************************************/

#include "ndk_demo.h"

using namespace NDK_XX;

/************************************************************************
*  	clase TestBox														*
*	A box we use to test the things we create... %-|					*
************************************************************************/
//-----------------------------------------------------------------------
const int ID_CLEAR=	10001;
const int ID_REMOVE=10002;
const int ID_EDIT=	10003;
const int ID_SELECTION=10004;
const int ID_CHOICE=10005;
const int ID_SINGLE_CHECK=10006;
const int ID_MULTICHECK=10007;


//-----------------------------------------------------------------------
BEGIN_BINDINGS(TestBox,NDKDialogBox)
	ON_NOTIFY(LI_SEL_CHANGE,ID_CHOICE,TestBox::on_sel_change)
	ON_COMMAND(ID_CLEAR,TestBox::on_clear_list)
	ON_COMMAND(ID_REMOVE,TestBox::on_remove)
END_BINDINGS()


//-----------------------------------------------------------------------
TestBox::TestBox(	const string &title
						,int lines, int cols,int y, int x)
	: NDKDialogBox(1,title,lines,cols,y,x,okCancel)
		,pEdit(0)
		,pSel(0)
		,pChoice(0)
		,pCheckList(0)
		,pCheckList2(0)
{
}


//-----------------------------------------------------------------------
void TestBox::on_dlg_init()
{
	string s1("Name:");
	add_child(new NDKDlgLabel(this,-1,s1,1,s1.length(),1,1));

	pEdit=new NDKDlgEdit(this,ID_EDIT,"",3,40,2,1);
	add_child(pEdit);
//	pCtrl->set_overwrite();
	
	int first = curr_child;
	
	add_child(new NDKDlgPushButton(this,ID_CLEAR,"Clear List",6,1));
	add_child(new NDKDlgPushButton(this,ID_REMOVE,"Remove",6,15));

	pSel = new NDKDlgSelectionBox(this,ID_SELECTION,"",6,10,8,1);
	pSel->set_vscrollbar();
	pSel->add_item(get_param("selection","1","First"));
	pSel->add_item(get_param("selection","2","Second"));
	pSel->add_item(get_param("selection","3","Third"));
	pSel->add_item(get_param("selection","4","Fourth"));
	pSel->add_item(get_param("selection","5","Fifth"));
	pSel->add_item(get_param("selection","6","Sixth"));
	pSel->add_item(get_param("selection","7","Seventh"));
	pSel->add_item(get_param("selection","8","Eighth"));
	pSel->add_item(get_param("selection","9","Nineth"));
	add_child(pSel);
	
	pChoice = new NDKDlgChoiceBox(this,ID_CHOICE,"",6,10,8,12);
	pChoice->add_item("First");
	pChoice->add_item("Last");
	pChoice->add_item("Middle");
	add_child(pChoice);
	refresh_choices();

	pChoice->set_cur_sel(get_param("default","4",0));
	
	pCheckList = new NDKDlgCheckListBox(this,ID_SINGLE_CHECK,"",6,10,8,23);
//	pCheckList->add_item("Check 1");
//	pCheckList->add_item("Check 2");
//	pCheckList->add_item("Check 3");
	add_child(pCheckList);
	
	pCheckList2 = new NDKDlgCheckListBox(this,ID_MULTICHECK,"",6,10,8,34);
	pCheckList2->add_item("Checkme");
	pCheckList2->add_item("Me too");
	pCheckList2->add_item("lemmein");
	pCheckList2->set_multisel();
	
	IndexList il;
	il.push_back(1);
	il.push_back(2);
	pCheckList2->set_sel_items(il);
	add_child(pCheckList2);

	create_buttons(); 
	set_focus_to(pEdit);
}


//-----------------------------------------------------------------------
void TestBox::on_clear_list()
{
	long curr = curr_child;
	pSel->clear();
	set_focus_to(curr);
}


//-----------------------------------------------------------------------
void TestBox::on_remove()
{
	long curr = curr_child;
	int choice_idx =pChoice->get_cur_sel();
	if(choice_idx == -1)
		return;
	int sel_idx = (int)pChoice->get_item_data(choice_idx);
	pSel->remove_item(sel_idx);
	refresh_choices();
	set_focus_to(curr);
	if(pBtnOk)
		pBtnOk->enable(); 
}


//-----------------------------------------------------------------------
void TestBox::refresh_choices()
{
	pChoice->set_item_data(0,0);
	pChoice->set_item_data(1,(void *)(pSel->get_count()-1));
	int pos =(pSel->get_count()/2)-1;
	if(pos < 0) pos=0;
	pChoice->set_item_data(2,(void *)(pos));
}


//-----------------------------------------------------------------------
void TestBox::on_sel_change(int id)
{
	int curr = curr_child;
	changed=true;
	if(pBtnOk)
		pBtnOk->enable();
	if(id == pChoice->get_ctrl_id()){
		int choice_idx =pChoice->get_cur_sel();
		int sel_idx = (int)pChoice->get_item_data(choice_idx);
		if(choice_idx < 0)
			return; 
		pSel->set_cur_sel(sel_idx);		
		pEdit->set_text(pSel->get_item_text(sel_idx));
	}
	set_focus_to(curr);
}


/************************************************************************
*	class APopUpCmd														*
************************************************************************/
//-----------------------------------------------------------------------
APopUpCmd::APopUpCmd()
	 : NDKPopUpMenu("PopUp")
{ 
	sub_menu_label = "PopUp";
	items.push_back(new BackCmd());
	items.push_back(new NonImplementedItem("Useless"));
	items.push_back(new ShowTestDialogCmd());
	items.push_back(new ClosingMarkItem());
}


/************************************************************************
*	class AnotherPopUpCmd												*
************************************************************************/
//-----------------------------------------------------------------------
AnotherPopUpCmd::AnotherPopUpCmd()
	: NDKPopUpMenu ("2nd Item")
{  
	sub_menu_label = "Second element";
	items.push_back(new BackCmd());
	items.push_back(new ShowTestDialogCmd());
	items.push_back(new NonImplementedItem("Second choice"));
	items.push_back(new NonImplementedItem("Third choice"));
	items.push_back(new NonImplementedItem("Fourth choice"));
	items.push_back(new NonImplementedItem("Fift Choice"));
	items.push_back(new ClosingMarkItem());
}


/************************************************************************
*	class TestApplication												*
************************************************************************/
//-----------------------------------------------------------------------
void TestApplication::title() {
  const char * const title = " NDK++ Demo Application";
  const int len = ::strlen(title);

  titleWindow->bkgd(screen_titles());
  titleWindow->addstr(0,(titleWindow->cols()-len)/2,title);
  titleWindow->refresh();
}


//-----------------------------------------------------------------------
int TestApplication::run() {
try{
	set_param_file_name("testapp.conf");
	activateStatusBar();

	ListOfItems items;
	items.push_back( new APopUpCmd());
	items.push_back( new AnotherPopUpCmd());
	items.push_back( new NonImplementedItem("Third"));
	items.push_back( new ExitCmd());
	items.push_back( new ClosingMarkItem());

	activateMenuBar(items);
	ListOfItems::iterator it;
	for(it=items.begin();it!=items.end();it++){
		delete *it;
	}
}
catch(NCursesException &e){
	cout 	<< e.classname() << ":" << e.errorno 
			<< "(" << e.message << ")" << endl;
}
catch(...){
	cout << "Unknown error" << endl;
}
	return 0;

}

//-----------------------------------------------------------------------
void TestApplication::activateMenuBar(ListOfItems &items)
{
	NDKMenuBar menuBar(1,80,0,0,items);
//	NDKFramedMenuBar menuBar(1,80,0,0,items);
	menuBar();
}

//-----------------------------------------------------------------------
static TestApplication testApp;
